// Fragment Shader  file "shadername.frag"

#version 150 core

//neuer shader
in vec3 LightDir;
in vec3 ViewDir;

//alter shader
in vec4 ex_color;
in vec3 ex_normal0;

//beide shader
in vec2 ex_texcoord0;
in float ex_hasTexture;
in float ex_isBump;

out vec4 FragColor;

struct DirectionalLight
{
    vec3 Color;
    float AmbientIntensity;
	float DiffuseIntensity;
	vec3 Direction;
};

uniform DirectionalLight gDirectionalLight;
uniform sampler2D gSampler;
uniform sampler2D gColorMapSampler;
uniform sampler2D gGlossMapSampler;
uniform sampler2D gNormalMapSampler;

uniform vec4 in_specular;

void main(void)
{
	if(ex_isBump > 0) {
		//neuer shader
		vec3 L = normalize(LightDir);
		vec3 N = normalize(texture(gNormalMapSampler, ex_texcoord0.st).xyz * 2.0 - 1.0);
		vec3 V = normalize(ViewDir);
		vec3 R = normalize(-reflect(L, N));

		float nDotL = max(0.0, dot(N, L));
		float rDotV = max(0.0, dot(R, V));
	
		vec4 ambient = vec4(gDirectionalLight.Color, 1.0f) * gDirectionalLight.AmbientIntensity;
		vec4 diffuse = vec4(gDirectionalLight.Color, 1.0f) * gDirectionalLight.DiffuseIntensity * nDotL;
		vec4 specular = in_specular * pow(rDotV, 60.0);
		vec4 texel = texture(gColorMapSampler, ex_texcoord0);
		float gloss = texture(gGlossMapSampler, ex_texcoord0).r;
		
		FragColor = (ambient + diffuse + (gloss * specular)) * texel;

	} else {
		//alter shader
		vec4 AmbientColor = vec4(gDirectionalLight.Color, 1.0f) * gDirectionalLight.AmbientIntensity; 
	
		float DiffuseFactor = dot(normalize(ex_normal0), -gDirectionalLight.Direction);
		vec4 DiffuseColor;
		if (DiffuseFactor > 0) {
			 DiffuseColor = vec4(gDirectionalLight.Color, 1.0f) * gDirectionalLight.DiffuseIntensity * DiffuseFactor;
		 
		} else {
			DiffuseColor = vec4(0.0f);
		}
	
		if(ex_hasTexture > 0) {
			FragColor = texture2D(gSampler, ex_texcoord0.xy) * ex_color * (AmbientColor + DiffuseColor);
		} else {
			FragColor = ex_color * (AmbientColor + DiffuseColor);
		
		}
	}
}
