// Vertex Shader  file "shadername.vert"

#version 150 core

uniform mat4 world_transformation_matrix;
uniform mat4 view_translation_matrix;
uniform mat4 view_rotation_matrix;
uniform mat4 object_transformation_matrix;
uniform mat4 perspective_matrix;
uniform mat3 NormalMatrix;

uniform float in_hasTexture;
uniform float in_isBump;

uniform vec3 direction;

in vec2 in_texcoord;
in vec3 in_position;
in vec4 in_color;
in vec3 in_normal;
in vec4 in_tangent;

//alter shader
out vec4 ex_color;
out vec3 ex_normal0;

//neuer shader
out vec3 LightDir;
out vec3 ViewDir;

//beide shader
out vec2 ex_texcoord0;
out float ex_hasTexture;
out float ex_isBump;

void main(void)
{
	if(in_isBump > 0) {
		//neuer shader
		vec3 n = normalize(NormalMatrix * in_normal);
		vec3 t = normalize(NormalMatrix * in_tangent.xyz);
		vec3 b = cross(n, t) * in_tangent.w;
    
		mat3 tbnMatrix = mat3(t.x, b.x, n.x,
							  t.y, b.y, n.y,
							  t.z, b.z, n.z);

		mat4 ViewMatrix =  view_translation_matrix * view_rotation_matrix * world_transformation_matrix * mat4(1.0f);

		mat4 modelViewMatrix = ViewMatrix * object_transformation_matrix;
		mat4 mvpMatrix = perspective_matrix * modelViewMatrix;
		
		vec4 pos = vec4(in_position.x, in_position.y, in_position.z, 1.0f);
    
		gl_Position = mvpMatrix * pos;
    
		pos = modelViewMatrix * pos;
		ViewDir = -(pos.xyz / pos.w);
		ViewDir = tbnMatrix * ViewDir;

		LightDir = vec3(ViewMatrix * vec4(-direction, 0.0f));
		LightDir = tbnMatrix * LightDir;
	} else {
		//alter shader
		gl_Position = perspective_matrix * view_translation_matrix * view_rotation_matrix * world_transformation_matrix * object_transformation_matrix * vec4(in_position, 1.0);
		ex_color = in_color;
		ex_normal0 = (world_transformation_matrix * object_transformation_matrix * vec4(in_normal, 0.0)).xyz;
	}
	
	//beide shader
	ex_texcoord0 = in_texcoord;
	ex_hasTexture = in_hasTexture;
	ex_isBump = in_isBump;
}
